@@ -17,9 +17,54 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gettext, gobject, gtk, locale, optparse, signal
+import dbus, optparse, sys
 
-from tools import consts, loadGladeFile, log, prefs
+from tools import consts
+
+
+# Command line
+optparser = optparse.OptionParser(usage='Usage: %prog [options] [FILE(s)]')
+optparser.add_option('-p', '--playbin2', action='store_true', default=False, help='use the playbin2 GStreamer component (unstable)')
+optparser.add_option('--no-glossy-cover', action='store_true', default=False, help='disable the gloss effect applied to covers')
+optparser.add_option('--multiple-instances', action='store_true', default=False, help='start a new instance even if one is already running')
+
+(optOptions, optArgs) = optparser.parse_args()
+
+
+# Check whether DAP is already running?
+if not optOptions.multiple_instances:
+    shouldStop  = False
+    dbusSession = None
+
+    try:
+        dbusSession    = dbus.SessionBus()
+        activeServices = dbusSession.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus').ListNames()
+
+        if consts.dbusService in activeServices:
+            shouldStop      = True
+            remoteObject    = dbusSession.get_object(consts.dbusService, consts.dbusObject)
+            remoteInterface = dbus.Interface(remoteObject, consts.dbusInterface)
+
+            # Fill the current instance with the given tracks, if any
+            if len(optArgs) != 0:
+                remoteInterface.sendMsg(consts.MSG_CMD_TRACKLIST_SET, optArgs)
+
+            # Bring the current instance to the front
+            remoteInterface.sendMsg(consts.MSG_CMD_BRING_TO_FRONT, '')
+    except:
+        pass
+
+    if dbusSession is not None:
+        dbusSession.close()
+
+    if shouldStop:
+        sys.exit(1)
+
+
+# Start a new instance
+import gettext, gobject, gtk, locale, signal
+
+from tools import loadGladeFile, log, prefs
 
 DEFAULT_VIEW_MODE       = consts.VIEW_MODE_FULL
 DEFAULT_PANED_POS       = 300
@@ -72,7 +117,6 @@ def realStartup():
     window.connect('window-state-event', onState)
     paned.connect('size-allocate', lambda win, rect: prefs.set(__name__, 'paned-pos', paned.get_position()))
     wTree.get_widget('menu-mode-mini').connect('activate', onViewMode, consts.VIEW_MODE_MINI)
-    wTree.get_widget('menu-mode-nano').connect('activate', onViewMode, consts.VIEW_MODE_NANO)
     wTree.get_widget('menu-mode-full').connect('activate', onViewMode, consts.VIEW_MODE_FULL)
     wTree.get_widget('menu-mode-playlist').connect('activate', onViewMode, consts.VIEW_MODE_PLAYLIST)
     wTree.get_widget('menu-quit').connect('activate', lambda item: onDelete(window, None))
@@ -104,8 +148,8 @@ def setViewMode(mode, resize):
         wTree.get_widget('scrolled-tracklist').show()
         wTree.get_widget('box-trkinfo').show()
         if resize:
-            if lastMode != consts.VIEW_MODE_FULL:                          winWidth  = winWidth + paned.get_position()
-            if lastMode in (consts.VIEW_MODE_MINI, consts.VIEW_MODE_NANO): winHeight = prefs.get(__name__, 'full-win-height', DEFAULT_WIN_HEIGHT)
+            if lastMode != consts.VIEW_MODE_FULL: winWidth  = winWidth + paned.get_position()
+            if lastMode == consts.VIEW_MODE_MINI: winHeight = prefs.get(__name__, 'full-win-height', DEFAULT_WIN_HEIGHT)
 
             window.resize(winWidth, winHeight)
         return
@@ -120,7 +164,7 @@ def setViewMode(mode, resize):
         wTree.get_widget('box-btn-tracklist').hide()
         wTree.get_widget('scrolled-tracklist').show()
         wTree.get_widget('box-trkinfo').show()
-        if resize and lastMode in (consts.VIEW_MODE_MINI, consts.VIEW_MODE_NANO):
+        if resize and lastMode == consts.VIEW_MODE_MINI:
             window.resize(winWidth, prefs.get(__name__, 'full-win-height', DEFAULT_WIN_HEIGHT))
         return
 
@@ -147,9 +191,7 @@ gtk.glade.textdomain(consts.appNameShort)
 gtk.glade.bindtextdomain(consts.appNameShort, consts.dirLocale)
 
 # Command line
-parser = optparse.OptionParser(usage='Usage: %prog [options] [FILE(s)]')
-parser.add_option('-p', '--playbin2', action='store_true', default=False, help='use the playbin2 GStreamer component (unstable)')
-prefs.setCmdLine(parser.parse_args())
+prefs.setCmdLine((optOptions, optArgs))
 
 # PyGTK initialization
 gobject.threads_init()
@@ -188,9 +230,8 @@ viewMode = prefs.get(__name__, 'view-mode', DEFAULT_VIEW_MODE)
 if viewMode == consts.VIEW_MODE_FULL:
     wTree.get_widget('menu-mode-full').set_active(True)
 else:
-    if viewMode == consts.VIEW_MODE_MINI:   wTree.get_widget('menu-mode-mini').set_active(True)
-    elif viewMode == consts.VIEW_MODE_NANO: wTree.get_widget('menu-mode-nano').set_active(True)
-    else:                                   wTree.get_widget('menu-mode-playlist').set_active(True)
+    if viewMode == consts.VIEW_MODE_MINI: wTree.get_widget('menu-mode-mini').set_active(True)
+    else:                                 wTree.get_widget('menu-mode-playlist').set_active(True)
 
     setViewMode(viewMode, False)
 
