@@ -16,7 +16,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 #
-# ExtListView v1.7
+# ExtListView v1.8
+#
+# v1.8:
+#   * The 'extlistview-modified' signal was not generated when calling clear() and replaceContent()
 #
 # v1.7:
 #   * (Hopefully) fixed the intermittent improper columns resizing
@@ -395,6 +398,7 @@ class ExtListView(gtk.TreeView):
         self.clearMark()
         self.store.clear()
         self.__resizeColumns()
+        self.emit('extlistview-modified')
 
 
     def setItem(self, rowIndex, colIndex, value):
@@ -561,7 +565,8 @@ class ExtListView(gtk.TreeView):
         if event.button == 1 or event.button == 3:
             if path is None:
                 self.selection.unselect_all()
-                tree.set_cursor(len(self.store))
+                if len(self.store) != 0:
+                    tree.set_cursor(len(self.store))
             else:
                 if self.dndReordering and self.motionEvtId is None and event.button == 1:
                     self.dndStartPos = (int(event.x), int(event.y))
