@@ -18,20 +18,20 @@
 
 import os, playlist, tools, traceback
 
-from format          import asf, flac, mp3, mp4, mpc, ogg
+from format          import monkeysaudio, asf, flac, mp3, mp4, mpc, ogg, wavpack
 from os.path         import splitext
 from tools.log       import logger
 from track.fileTrack import FileTrack
 
 
 # Supported formats with associated modules
-mFormats = {'.flac': flac, '.m4a': mp4, '.mp3': mp3, '.mp4': mp4, '.mpc': mpc,'.ogg': ogg, '.wma': asf}
+mFormats = {'.ac3': monkeysaudio, '.ape': monkeysaudio, '.flac': flac, '.m4a': mp4, '.mp2': mp3, '.mp3': mp3, '.mp4': mp4, '.mpc': mpc,'.oga': ogg, '.ogg': ogg, '.wma': asf, '.wv': wavpack}
 
 
 def isSupported(file):
     """ Return True if the given file is a supported format """
-    file = file.lower()
-    return file[-4:] in mFormats or file[-5:] in mFormats
+    try:    return splitext(file.lower())[1] in mFormats
+    except: return False
 
 
 def getSupportedFormats():
@@ -44,11 +44,8 @@ def getTrackFromFile(file):
         Return a Track object, based on the tags of the given file
         The 'file' parameter must be a real file (not a playlist or a directory)
     """
-    file2 = file.lower()
-
     try:
-        try:    return mFormats[file2[-4:]].getTrack(file)
-        except: return mFormats[file2[-5:]].getTrack(file)
+        return mFormats[splitext(file.lower())[1]].getTrack(file)
     except:
         logger.error('Unable to extract information from %s\n\n%s' % (file, traceback.format_exc()))
         return FileTrack(file)
@@ -59,26 +56,29 @@ def getTracksFromFiles(files):
     return [getTrackFromFile(file) for file in files]
 
 
-def getTracks(filenames):
+def getTracks(filenames, sortByFilename=False):
     """ Same as getTracksFromFiles(), but works for any kind of filenames (files, playlists, directories) """
     allTracks = []
 
     # Directories
     for directory in [filename for filename in filenames if os.path.isdir(filename)]:
-        files, playlists = [], []
-        for (filename, path) in tools.listDir(directory):
-            if os.path.isfile(path):
-                if isSupported(filename):            files.append(path)
-                elif playlist.isSupported(filename): playlists.append(path)
+        mediaFiles, playlists = [], []
+        for root, subdirs, files in os.walk(directory):
+            for file in files:
+                if isSupported(file):            mediaFiles.append(os.path.join(root, file))
+                elif playlist.isSupported(file): playlists.append(os.path.join(root, file))
 
-        allTracks.extend(sorted(getTracksFromFiles(files)))
+        if sortByFilename: allTracks.extend(sorted(getTracksFromFiles(mediaFiles), lambda t1, t2: cmp(t1.getFilePath(), t2.getFilePath())))
+        else:              allTracks.extend(sorted(getTracksFromFiles(mediaFiles)))
 
         for pl in playlists:
             allTracks.extend(getTracksFromFiles(playlist.load(pl)))
 
     # Files
     tracks = getTracksFromFiles([filename for filename in filenames if os.path.isfile(filename) and isSupported(filename)])
-    allTracks.extend(sorted(tracks))
+
+    if sortByFilename: allTracks.extend(sorted(tracks, lambda t1, t2: cmp(t1.getFilePath(), t2.getFilePath())))
+    else:              allTracks.extend(sorted(tracks))
 
     # Playlists
     for pl in [filename for filename in filenames if os.path.isfile(filename) and playlist.isSupported(filename)]:
