@@ -26,10 +26,11 @@ from media.track.cdTrack import CDTrack
 
 MOD_INFO = ('Audio CD', _('Audio CD'), _('Play audio discs'), ('DiscID', 'CDDB'), False, True)
 
-MOD_L10N            = MOD_INFO[modules.MODINFO_L10N]
-PREFS_DFT_DEVICE    = '/dev/cdrom'
-PREFS_DFT_USE_CDDB  = True
-PREFS_DFT_USE_CACHE = True
+MOD_L10N             = MOD_INFO[modules.MODINFO_L10N]
+PREFS_DFT_DEVICE     = '/dev/cdrom'
+PREFS_DFT_USE_CDDB   = True
+PREFS_DFT_USE_CACHE  = True
+PREFS_DFT_READ_SPEED = 1
 
 
 # Format of a row in the treeview
@@ -41,6 +42,26 @@ PREFS_DFT_USE_CACHE = True
 ) = range(4)
 
 
+# All CD-ROM read speeds
+READ_SPEEDS = {
+                 1 :  0,
+                 2 :  1,
+                 4 :  2,
+                 8 :  3,
+                10 :  4,
+                12 :  5,
+                20 :  6,
+                32 :  7,
+                36 :  8,
+                40 :  9,
+                48 : 10,
+                50 : 11,
+                52 : 12,
+                56 : 13,
+                72 : 14,
+              }
+
+
 # Information returned by disc_id()
 DISC_FRAME1    =  2
 DISC_FRAMEn    = -2
@@ -85,9 +106,10 @@ class AudioCD(modules.ThreadedModule):
         self.tree.connect('key-press-event',            self.onKeyPressed)
         self.tree.connect('exttreeview-button-pressed', self.onButtonPressed)
         modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': self.expName, 'icon': consts.icoCdrom, 'widget': self.scrolled})
-
         # Hide the album length when not drawing the root node
         self.tree.get_column(0).set_cell_data_func(txtRdrLen, self.__drawAlbumLenCell)
+        # CD-ROM drive read speed
+        modules.postMsg(consts.MSG_CMD_SET_CD_SPEED, {'speed': prefs.get(__name__, 'read-speed', PREFS_DFT_READ_SPEED)})
 
 
     def __drawAlbumLenCell(self, column, cell, model, iter):
@@ -122,10 +144,11 @@ class AudioCD(modules.ThreadedModule):
             Replace/extend the tracklist
             If the list 'paths' is None, use the current selection
         """
-        tracks = self.getTracksFromPaths(tree, paths)
+        if self.tree.getNbChildren((0,)) != 0:
+            tracks = self.getTracksFromPaths(tree, paths)
 
-        if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
-        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
+            if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
+            else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
 
 
     # --== Cache management ==--
@@ -256,7 +279,7 @@ class AudioCD(modules.ThreadedModule):
             discInfo = DiscID.disc_id(DiscID.open(prefs.get(__name__, 'device', PREFS_DFT_DEVICE)))
         except Exception, err:
             if err[0] == 123:
-                self.tree.replaceContent([(consts.icoCdrom, _('No disc found'), None)])
+                self.tree.replaceContent([(consts.icoCdrom, None, _('No disc found'), None)])
                 modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME, {'modName': MOD_L10N, 'expName': self.expName, 'newExpName': MOD_L10N})
                 self.expName = MOD_L10N
             else:
@@ -309,6 +332,14 @@ class AudioCD(modules.ThreadedModule):
             self.popup.get_widget('item-play').connect('activate',    lambda widget: self.playPaths(tree, None, True))
             self.popup.get_widget('item-refresh').connect('activate', lambda widget: self.reloadDisc())
 
+        # Enable/disable menu entries depending on whether there is something to play
+        if self.tree.getNbChildren((0,)) == 0:
+            self.popup.get_widget('item-add').set_sensitive(False)
+            self.popup.get_widget('item-play').set_sensitive(False)
+        else:
+            self.popup.get_widget('item-add').set_sensitive(True)
+            self.popup.get_widget('item-play').set_sensitive(True)
+
         self.popup.get_widget('menu-popup').popup(None, None, None, button, time)
 
 
@@ -349,12 +380,26 @@ class AudioCD(modules.ThreadedModule):
             self.cfgWin.getWidget('btn-clearCache').connect('clicked', self.onBtnClearCache)
             self.cfgWin.getWidget('btn-cancel').connect('clicked', lambda btn: self.cfgWin.hide())
 
+            # Set up the combo box
+            combo = self.cfgWin.getWidget('combo-read-speed')
+            txtRenderer = gtk.CellRendererText()
+            combo.pack_start(txtRenderer, True)
+            combo.add_attribute(txtRenderer, 'text', 0)
+            combo.set_sensitive(True)
+            txtRenderer.set_property('xpad', 6)
+            # Setup the liststore
+            store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
+            combo.set_model(store)
+            for speed in sorted(READ_SPEEDS.iterkeys()):
+                store.append(('%ux' % speed, speed))
+
         if not self.cfgWin.isVisible():
             self.cfgWin.getWidget('btn-ok').grab_focus()
             self.cfgWin.getWidget('txt-device').set_text(prefs.get(__name__, 'device', PREFS_DFT_DEVICE))
             self.cfgWin.getWidget('chk-useCDDB').set_active(prefs.get(__name__, 'use-cddb', PREFS_DFT_USE_CDDB))
             self.cfgWin.getWidget('chk-useCache').set_sensitive(prefs.get(__name__, 'use-cddb', PREFS_DFT_USE_CDDB))
             self.cfgWin.getWidget('chk-useCache').set_active(prefs.get(__name__, 'use-cache', PREFS_DFT_USE_CACHE))
+            self.cfgWin.getWidget('combo-read-speed').set_active(READ_SPEEDS[prefs.get(__name__, 'read-speed', PREFS_DFT_READ_SPEED)])
 
         self.cfgWin.show()
 
@@ -375,9 +420,10 @@ class AudioCD(modules.ThreadedModule):
 
     def onBtnOk(self, btn):
         """ Check that entered information is correct before saving everything """
-        device   = self.cfgWin.getWidget('txt-device').get_text()
-        useCDDB  = self.cfgWin.getWidget('chk-useCDDB').get_active()
-        useCache = useCDDB and self.cfgWin.getWidget('chk-useCache').get_active()
+        device    = self.cfgWin.getWidget('txt-device').get_text()
+        useCDDB   = self.cfgWin.getWidget('chk-useCDDB').get_active()
+        useCache  = useCDDB and self.cfgWin.getWidget('chk-useCache').get_active()
+        readSpeed = self.cfgWin.getWidget('combo-read-speed').get_model()[self.cfgWin.getWidget('combo-read-speed').get_active()][1]
 
         if not os.path.exists(device):
             error    = _('Invalid path')
@@ -385,11 +431,15 @@ class AudioCD(modules.ThreadedModule):
             gui.errorMsgBox(self.cfgWin, error, errorMsg)
             self.cfgWin.getWidget('txt-device').grab_focus()
         else:
-            prefs.set(__name__, 'device',    device)
-            prefs.set(__name__, 'use-cddb',  useCDDB)
-            prefs.set(__name__, 'use-cache', useCache)
+            prefs.set(__name__, 'device',     device)
+            prefs.set(__name__, 'use-cddb',   useCDDB)
+            prefs.set(__name__, 'use-cache',  useCache)
+            prefs.set(__name__, 'read-speed', readSpeed)
             self.cfgWin.hide()
 
+            # CD-ROM drive read speed
+            modules.postMsg(consts.MSG_CMD_SET_CD_SPEED, {'speed': readSpeed})
+
 
     def onBtnHelp(self, btn):
         """ Display a small help message box """
