@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import md5, modules, os.path, socket, tools, traceback
+import hashlib, modules, os.path, socket, tools, traceback
 
 from gui       import authentication
 from time      import time, sleep
@@ -160,7 +160,15 @@ class AudioScrobbler(modules.ThreadedModule):
         if self.passwd is None:
             return False
 
-        token              = md5.md5('%s%d' % (md5.md5(self.passwd).hexdigest(), now)).hexdigest()
+        # Compute the authentication token
+        md5Pwd   = hashlib.md5()
+        md5Token = hashlib.md5()
+
+        md5Pwd.update(self.passwd)
+        md5Token.update('%s%u' % (md5Pwd.hexdigest(), now))
+
+        # Try to forget authentication info ASAP
+        token              = md5Token.hexdigest()
         self.passwd        = None
         request            = 'http://%s/?hs=true&p=%s&c=%s&v=%s&u=%s&t=%d&a=%s' % (AS_SERVER, PROTO_VER, CLI_ID, CLI_VER, self.login, now, token)
         self.lastHandshake = now
@@ -267,24 +275,24 @@ class AudioScrobbler(modules.ThreadedModule):
         return False
 
 
-    def onTrackEnded(self):
+    def onTrackEnded(self, trySubmission):
         """ The playback of the current track has stopped """
         if self.currTrack is not None:
             self.currTrack[TRK_PLAY_TIME] += (int(time()) - self.currTrack[TRK_UNPAUSED_TIMESTAMP])
             self.addToCache()
-
-            # Try to submit the whole cache
-            submitOk = self.submit()
-            while submitOk and len(self.cache) != 0:
-                submitOk = self.submit()
-
             self.currTrack = None
 
+            # Try to submit the whole cache?
+            if trySubmission:
+                submitOk = self.submit()
+                while submitOk and len(self.cache) != 0:
+                    submitOk = self.submit()
+
 
     def onNewTrack(self, track):
         """ A new track has started """
         timestamp = int(time())
-        self.onTrackEnded()
+        self.onTrackEnded(True)
         self.nowPlayingNotification(track)
         self.currTrack = [timestamp, timestamp, 0, track]
 
@@ -301,7 +309,7 @@ class AudioScrobbler(modules.ThreadedModule):
             if self.paused:
                 self.currTrack[TRK_UNPAUSED_TIMESTAMP] = int(time())
             self.paused = False
-            self.onTrackEnded()
+            self.onTrackEnded(True)
 
         elif msg == consts.MSG_EVT_PAUSED:
             self.currTrack[TRK_PLAY_TIME] += (int(time()) - self.currTrack[TRK_UNPAUSED_TIMESTAMP])
@@ -315,7 +323,7 @@ class AudioScrobbler(modules.ThreadedModule):
             if self.paused:
                 self.currTrack[TRK_UNPAUSED_TIMESTAMP] = int(time())
             self.paused = False
-            self.addToCache()
+            self.onTrackEnded(False)
             self.saveCache()
             if self.getCacheSize() != 0:
                 logger.info('[%s] %u track(s) left in cache' % (MOD_NAME, self.getCacheSize()))
