@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gtk.gdk, gui, modules, os, socket, tempfile, tools, traceback, urllib2
+import gui, Image, modules, os, socket, tempfile, tools, traceback, urllib2
 
 from tools     import consts, prefs
 from gettext   import gettext as _
@@ -42,10 +42,10 @@ USER_AGENT = 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/20080728
 ) = range(2)
 
 # Constants for thumbnails
-THUMBNAIL_WIDTH   = 74   # Width allocated to thumbnails in the model
-THUMBNAIL_HEIGHT  = 74   # Height allocated to thumbnails in the model
-THUMBNAIL_OFFSETX =  8   # X-offset to render the thumbnail in the model
-THUMBNAIL_OFFSETY =  0   # Y-offset to render the thumbnail in the model
+THUMBNAIL_WIDTH   = 100  # Width allocated to thumbnails in the model
+THUMBNAIL_HEIGHT  = 100  # Height allocated to thumbnails in the model
+THUMBNAIL_OFFSETX =  11  # X-offset to render the thumbnail in the model
+THUMBNAIL_OFFSETY =   3  # Y-offset to render the thumbnail in the model
 
 # Constants for full size covers
 FULL_SIZE_COVER_WIDTH  = 300
@@ -57,10 +57,11 @@ ACCEPTED_FILE_FORMATS = {'.jpg': None, '.jpeg': None, '.png': None, '.gif': None
 # Default preferences
 PREFS_DFT_DOWNLOAD_COVERS      = False
 PREFS_DFT_PREFER_USER_COVERS   = True
-PREFS_DFT_USER_COVER_FILENAMES = ['album', 'art', 'cover', 'front']
+PREFS_DFT_USER_COVER_FILENAMES = ['cover', 'art', 'front', '*']
 
-# Since it's very small, we integrate the PNG file directly in the source code
-THUMBNAIL_MODEL = '\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00R\x00\x00\x00J\x08\x00\x00\x00\x00\r\xc2\\\xb1\x00\x00\x00\x02bKGD\x00\xff\x87\x8f\xcc\xbf\x00\x00\x00vIDATX\xc3\xed\x97\xd1\r\x80@\x08C\x1bFq\x08\xf7qOg\x13s\x9e\x1b\xc8%\x92\xbe\xf2\xdf\x10z\xb4G\x1c\x13{\x96!\xce\x07*Dl\x1aUJ\xa9r4\xa1\x9c*)\x7f\xde\xa5J[\xb4V\xdc\x96\xf2\xcaQ\x99\xcc\xd2\xc6\xdcZ\xd8\xc6\x9b\x94(\x0e\xe5\xe7GD\x8e\x9b\xc5\x99\x883\x9b\xa7\xdec\xc7\xdds\x1cy8\x9e\xf1K\x0e>v\x9c/+\xe6\xc6,\xd7\x9a[!\xe5\r(\xf6\xc6u\xfe\xcbvu\x00\x00\x00\x00IEND\xaeB`\x82'
+# Images for thumbnails
+THUMBNAIL_GLOSS = os.path.join(consts.dirPix, 'cover-gloss.png')
+THUMBNAIL_MODEL = os.path.join(consts.dirPix, 'cover-model.png')
 
 
 class Covers(modules.ThreadedModule):
@@ -73,12 +74,11 @@ class Covers(modules.ThreadedModule):
 
     def onModLoaded(self):
         """ The module has been loaded """
-        self.cfgWin             = None                                   # Configuration window
-        self.coverMap           = {}                                     # Store covers previously requested
-        self.currTrack          = None                                   # The current track being played, if any
-        self.cacheRootPath      = os.path.join(consts.dirCfg, MOD_NAME)  # Local cache for Internet covers
-        self.coverBlacklist     = {}                                     # When a cover cannot be downloaded, avoid requesting it again
-        self.thumbnailModelPath = None                                   # Path to the model we use to generate the covers
+        self.cfgWin         = None                                   # Configuration window
+        self.coverMap       = {}                                     # Store covers previously requested
+        self.currTrack      = None                                   # The current track being played, if any
+        self.cacheRootPath  = os.path.join(consts.dirCfg, MOD_NAME)  # Local cache for Internet covers
+        self.coverBlacklist = {}                                     # When a cover cannot be downloaded, avoid requesting it again
 
         if not os.path.exists(self.cacheRootPath):
             os.mkdir(self.cacheRootPath)
@@ -97,101 +97,86 @@ class Covers(modules.ThreadedModule):
                 os.remove(covers[CVR_FULL])
         self.coverMap = None
 
-        # Delete the cover model if any
-        if self.thumbnailModelPath is not None and os.path.exists(self.thumbnailModelPath):
-            os.remove(self.thumbnailModelPath)
-        self.thumbnailModelPath = None
-
         # Delete blacklist
         self.coverBlacklist = None
 
 
-    def generateFullSizeCover(self, inFile, outFile):
+    def generateFullSizeCover(self, inFile, outFile, format):
         """ Resize inFile if needed, and write it to outFile (outFile and inFile may be equal) """
         try:
-            cover = gtk.gdk.pixbuf_new_from_file(inFile)
+            # Open the image
+            cover = Image.open(inFile)
 
-            # Resize values
-            if cover.get_width() <= FULL_SIZE_COVER_WIDTH:
-                scaleX    = 1.0
-                destWidth = cover.get_width()
-            else:
-                scaleX    = float(FULL_SIZE_COVER_WIDTH) / float(cover.get_width())
-                destWidth = FULL_SIZE_COVER_WIDTH
+            # Resize in the best way we can
+            if cover.size[0] < FULL_SIZE_COVER_WIDTH: newWidth = cover.size[0]
+            else:                                     newWidth = FULL_SIZE_COVER_WIDTH
 
-            if cover.get_height() <= FULL_SIZE_COVER_HEIGHT:
-                scaleY     = 1.0
-                destHeight = cover.get_height()
-            else:
-                scaleY     = float(FULL_SIZE_COVER_HEIGHT) / float(cover.get_height())
-                destHeight = FULL_SIZE_COVER_HEIGHT
+            if cover.size[1] < FULL_SIZE_COVER_HEIGHT: newHeight = cover.size[1]
+            else:                                      newHeight = FULL_SIZE_COVER_HEIGHT
 
-            if scaleX != 1 or scaleY != 1:
-                # Alpha channel is needed to prevent errors when the source cover has one
-                fullSizeCover = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, destWidth, destHeight)
-                cover.scale(fullSizeCover, 0, 0, destWidth, destHeight, 0, 0, scaleX, scaleY, gtk.gdk.INTERP_HYPER)
-                fullSizeCover.save(outFile, 'jpeg', {'quality':'100'})
-            else:
-                cover.save(outFile, 'jpeg', {'quality':'100'})
+            cover = cover.resize((newWidth, newHeight), Image.ANTIALIAS)
+
+            # We're done
+            cover.save(outFile, format)
         except:
             logger.error('[%s] An error occurred while generating a showable full size cover\n\n%s' % (MOD_NAME, traceback.format_exc()))
 
 
-    def generateThumbnail(self, inFile, outFile):
+    def generateThumbnail(self, inFile, outFile, format):
         """ Generate a thumbnail from inFile (e.g., resize it) and write it to outFile (outFile and inFile may be equal) """
         try:
-            cover = gtk.gdk.pixbuf_new_from_file(inFile)
+            # Open the image
+            cover = Image.open(inFile).convert('RGBA')
 
-            # Resize values
-            if cover.get_width() <= THUMBNAIL_WIDTH:
-                destX     = (THUMBNAIL_WIDTH - cover.get_width()) / 2 + THUMBNAIL_OFFSETX
-                scaleX    = 1.0
-                offsetX   = (THUMBNAIL_WIDTH - cover.get_width()) / 2 + THUMBNAIL_OFFSETX
-                destWidth = THUMBNAIL_WIDTH
+            # Resize in the best way we can
+            if cover.size[0] < THUMBNAIL_WIDTH:
+                newWidth = cover.size[0]
+                offsetX  = (THUMBNAIL_WIDTH - cover.size[0]) / 2
             else:
-                destX     = THUMBNAIL_OFFSETX
-                scaleX    = float(THUMBNAIL_WIDTH) / float(cover.get_width())
-                offsetX   = THUMBNAIL_OFFSETX
-                destWidth = THUMBNAIL_WIDTH
+                newWidth = THUMBNAIL_WIDTH
+                offsetX  = 0
 
-            if cover.get_height() <= THUMBNAIL_HEIGHT:
-                destY      = (THUMBNAIL_HEIGHT - cover.get_height()) / 2 + THUMBNAIL_OFFSETY
-                scaleY     = 1.0
-                offsetY    = (THUMBNAIL_HEIGHT - cover.get_height()) / 2 + THUMBNAIL_OFFSETY
-                destHeight = THUMBNAIL_HEIGHT
+            if cover.size[1] < THUMBNAIL_HEIGHT:
+                newHeight = cover.size[1]
+                offsetY   = (THUMBNAIL_HEIGHT - cover.size[1]) / 2
             else:
-                destY      = THUMBNAIL_OFFSETY
-                scaleY     = float(THUMBNAIL_HEIGHT) / float(cover.get_height())
-                offsetY    = THUMBNAIL_OFFSETY
-                destHeight = THUMBNAIL_HEIGHT
+                newHeight = THUMBNAIL_HEIGHT
+                offsetY   = 0
 
-            # Generate the model we use, if needed
-            if self.thumbnailModelPath is None or not os.path.exists(self.thumbnailModelPath):
-                self.thumbnailModelPath = tempfile.mktemp() + '.png'
-                output = open(self.thumbnailModelPath, 'wb')
-                output.write(THUMBNAIL_MODEL)
-                output.close()
+            cover = cover.resize((newWidth, newHeight), Image.ANTIALIAS)
 
-            # Alpha channel is needed to prevent errors when the source cover has one
-            coverModel = gtk.gdk.pixbuf_new_from_file(self.thumbnailModelPath).add_alpha(False, chr(255), chr(255), chr(255))
-            cover.scale(coverModel, destX, destY, destWidth, destHeight, offsetX, offsetY, scaleX, scaleY, gtk.gdk.INTERP_HYPER)
-            coverModel.save(outFile, 'jpeg', {'quality':'100'})
+            # Paste the resized cover into our model
+            model = Image.open(THUMBNAIL_MODEL).convert('RGBA')
+            model.paste(cover, (THUMBNAIL_OFFSETX + offsetX, THUMBNAIL_OFFSETY + offsetY), cover)
+            cover = model
+
+            # Don't apply the gloss effect if asked to
+            if not prefs.getCmdLine()[0].no_glossy_cover:
+                gloss = Image.open(THUMBNAIL_GLOSS).convert('RGBA')
+                cover.paste(gloss, (0, 0), gloss)
+
+            # We're done
+            cover.save(outFile, format)
         except:
             logger.error('[%s] An error occurred while generating a thumbnail\n\n%s' % (MOD_NAME, traceback.format_exc()))
 
 
     def getUserCover(self, trackPath):
-        """
-            Check whether a user cover (e.g., cover.jpg) exists in the given directory:
-                * If so, return the path to that file
-                * Otherwise, return None
-        """
-        userCoverFilenames = prefs.get(__name__, 'user-cover-filenames', PREFS_DFT_USER_COVER_FILENAMES)
-
+        """ Return the path to a cover file in trackPath, None if no cover found """
+        # Create a dictionary with candidates
+        candidates = {}
         for (file, path) in tools.listDir(trackPath, True):
             (name, ext) = os.path.splitext(file.lower())
-            if ext in ACCEPTED_FILE_FORMATS and name in userCoverFilenames:
-                return path
+            if ext in ACCEPTED_FILE_FORMATS:
+                candidates[name] = path
+
+        # Check each possible name using the its index in the list as its priority
+        for name in prefs.get(__name__, 'user-cover-filenames', PREFS_DFT_USER_COVER_FILENAMES):
+            if name in candidates:
+                return candidates[name]
+
+            if name == '*' and len(candidates) != 0:
+                return candidates.values()[0]
 
         return None
 
@@ -354,10 +339,10 @@ class Covers(modules.ThreadedModule):
         # If we still don't have a cover, too bad
         # Otherwise, generate a thumbnail and a full size cover, and add it to our cover map
         if rawCover is not None:
-            thumbnail     = tempfile.mktemp() + '.jpg'
-            fullSizeCover = tempfile.mktemp() + '.jpg'
-            self.generateThumbnail(rawCover, thumbnail)
-            self.generateFullSizeCover(rawCover, fullSizeCover)
+            thumbnail     = tempfile.mktemp() + '.png'
+            fullSizeCover = tempfile.mktemp() + '.png'
+            self.generateThumbnail(rawCover, thumbnail, 'PNG')
+            self.generateFullSizeCover(rawCover, fullSizeCover, 'PNG')
             if os.path.exists(thumbnail) and os.path.exists(fullSizeCover):
                 self.coverMap[coverKey] = (thumbnail, fullSizeCover)
                 modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': thumbnail, 'pathFullSize': fullSizeCover})
