@@ -56,14 +56,13 @@ class CtrlPanel(modules.Module):
         self.onStopped()
         self.btnPlay.set_sensitive(False)
         # GTK handlers
-        self.btnStop.connect('clicked',      lambda widget: modules.postMsg(consts.MSG_CMD_STOP))
-        self.btnNext.connect('clicked',      lambda widget: modules.postMsg(consts.MSG_CMD_NEXT))
-        self.btnPrev.connect('clicked',      lambda widget: modules.postMsg(consts.MSG_CMD_PREVIOUS))
-        self.btnPlay.connect('clicked',      lambda widget: modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE))
-        self.sclSeek.connect('change-value', self.onSeekChangeValue)
-        # We need to keep the handler ID for this one to be able to disconnect it when needed
-        self.seekHandler   = self.sclSeek.connect('value-changed',   self.onSeekValueChanged)
-        self.volumeHandler = self.btnVolume.connect('value-changed', self.onVolumeValueChanged)
+        self.btnStop.connect('clicked',         lambda widget: modules.postMsg(consts.MSG_CMD_STOP))
+        self.btnNext.connect('clicked',         lambda widget: modules.postMsg(consts.MSG_CMD_NEXT))
+        self.btnPrev.connect('clicked',         lambda widget: modules.postMsg(consts.MSG_CMD_PREVIOUS))
+        self.btnPlay.connect('clicked',         lambda widget: modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE))
+        self.sclSeek.connect('change-value',    self.onSeekChangeValue)
+        self.sclSeek.connect('value-changed',   self.onSeekValueChanged)
+        self.btnVolume.connect('value-changed', self.onVolumeValueChanged)
         # We must make sure that the handler will be called: this is not the case if the new value is the same as the old one
         volumeValue = prefs.get(__name__, 'volume', PREFS_DEFAULT_VOLUME)
         if self.btnVolume.get_value() != volumeValue: self.btnVolume.set_value(volumeValue)
@@ -75,7 +74,7 @@ class CtrlPanel(modules.Module):
         self.btnStop.set_sensitive(True)
         self.btnPlay.set_sensitive(True)
         self.btnPlay.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON))
-        gtk.tooltips_data_get(self.btnPlay)[0].set_tip(self.btnPlay, _('Pause the current track'))
+        self.btnPlay.set_tooltip_text(_('Pause the current track'))
         self.currTrackLength = track.getLength()
         self.sclSeek.show()
         self.lblElapsed.show()
@@ -92,7 +91,7 @@ class CtrlPanel(modules.Module):
         self.btnNext.set_sensitive(False)
         self.btnPrev.set_sensitive(False)
         self.btnPlay.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_BUTTON))
-        gtk.tooltips_data_get(self.btnPlay)[0].set_tip(self.btnPlay, _('Play the first track of the playlist'))
+        self.btnPlay.set_tooltip_text(_('Play the first track of the playlist'))
         self.sclSeek.hide()
         self.lblElapsed.hide()
         self.lblRemaining.hide()
@@ -106,16 +105,16 @@ class CtrlPanel(modules.Module):
                 elapsedTime = self.currTrackLength
             self.lblRemaining.set_label(sec2str(self.currTrackLength - elapsedTime))
             # Make sure the handler will not be called
-            self.sclSeek.disconnect(self.seekHandler)
+            self.sclSeek.handler_block_by_func(self.onSeekValueChanged)
             self.sclSeek.set_value(elapsedTime)
-            self.seekHandler = self.sclSeek.connect('value-changed', self.onSeekValueChanged)
+            self.sclSeek.handler_unblock_by_func(self.onSeekValueChanged)
 
 
     def onVolumeChanged(self, value):
         """ The volume has been changed """
-        self.btnVolume.disconnect(self.volumeHandler)
+        self.btnVolume.handler_block_by_func(self.onVolumeValueChanged)
         self.btnVolume.set_value(value)
-        self.volumeHandler = self.btnVolume.connect('value-changed', self.onVolumeValueChanged)
+        self.btnVolume.handler_unblock_by_func(self.onVolumeValueChanged)
 
 
     def onTrackMoved(self, hasPrevious, hasNext):
