@@ -22,12 +22,13 @@ from media     import track
 from tools     import consts, prefs
 from gettext   import gettext as _
 from tools.log import logger
+from threading import Timer
 
 MOD_INFO = ('Desktop Notification', _('Desktop Notification'), _('Display a desktop notification on track change'), ['pynotify'], False, True)
 MOD_L10N = MOD_INFO[modules.MODINFO_L10N]
 
 # Default preferences
-PREFS_DEFAULT_BODY       = 'by {artist}\nfrom {album}\n\nTrack {playlist_pos} out of {playlist_len}'
+PREFS_DEFAULT_BODY       = 'by {artist} on {album} ({playlist_pos} / {playlist_len})'
 PREFS_DEFAULT_TITLE      = '{title}  [{duration_str}]'
 PREFS_DEFAULT_TIMEOUT    = 10
 PREFS_DEFAULT_SKIP_TRACK = False
@@ -39,7 +40,7 @@ class DesktopNotification(modules.ThreadedModule):
         """ Constructor """
         modules.ThreadedModule.__init__(self, (consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_MOD_LOADED,   consts.MSG_EVT_NEW_TRACK,
                                                consts.MSG_EVT_APP_QUIT,    consts.MSG_EVT_MOD_UNLOADED, consts.MSG_EVT_STOPPED,
-                                               consts.MSG_EVT_TRACK_MOVED))
+                                               consts.MSG_EVT_TRACK_MOVED, consts.MSG_CMD_SET_COVER))
 
 
     def initPynotify(self):
@@ -51,37 +52,47 @@ class DesktopNotification(modules.ThreadedModule):
 
     def onModLoaded(self):
         """ The module has been loaded """
-        self.icon        = gtk.STOCK_DIALOG_INFO
+        self.timer       = None
         self.notif       = None
         self.cfgWin      = None
         self.hasNext     = False
+        self.currTrack   = None
+        self.currCover   = None
         self.initialized = False
 
         if self.gtkExecute(self.initPynotify): self.initialized = True
         else:                                  logger.error('[%s] Initialization failed' % MOD_INFO[modules.MODINFO_NAME])
 
 
-    def showNotification(self, track):
-        """ Show the notification based on the given track """
-        body  = track.formatHTMLSafe(prefs.get(__name__, 'body',  PREFS_DEFAULT_BODY))
-        title = track.format(prefs.get(__name__, 'title', PREFS_DEFAULT_TITLE))
+    def showNotification(self):
+        """ Show the notification based on the current track """
+        self.timer = None
+
+        if self.currTrack is None:
+            return
+
+        body  = self.currTrack.formatHTMLSafe(prefs.get(__name__, 'body',  PREFS_DEFAULT_BODY))
+        title = self.currTrack.format(prefs.get(__name__, 'title', PREFS_DEFAULT_TITLE))
 
         # Make sure the notification exists
         if self.notif is None:
             import pynotify
 
-            img = os.path.join(consts.dirPix, 'decibel-audio-player-64.png')
-            if os.path.isfile(img):
-                self.icon = 'file://' + img
-
-            self.notif = pynotify.Notification(title, body, self.icon)
+            self.notif = pynotify.Notification(title, body, gtk.STOCK_DIALOG_INFO)
             self.notif.set_urgency(pynotify.URGENCY_LOW)
             self.notif.set_timeout(prefs.get(__name__, 'timeout', PREFS_DEFAULT_TIMEOUT) * 1000)
 
             if prefs.get(__name__, 'skip-track', PREFS_DEFAULT_SKIP_TRACK):
                 self.notif.add_action('stop', _('Skip track'), self.onSkipTrack)
 
-        self.notif.update(title, body, self.icon)
+        # Which image to display?
+        if self.currCover is None: img = os.path.join(consts.dirPix, 'decibel-audio-player-64.png')
+        else:                      img = self.currCover
+
+        if os.path.isfile(img): icon = 'file://' + img
+        else:                   icon = gtk.STOCK_DIALOG_INFO
+
+        self.notif.update(title, body, icon)
         self.notif.show()
 
 
@@ -91,19 +102,40 @@ class DesktopNotification(modules.ThreadedModule):
         else:            modules.postMsg(consts.MSG_CMD_STOP)
 
 
+    def __stopTimer(self):
+        """ Stop the timer if any """
+        if self.timer is not None:
+            self.timer.cancel()
+            self.timer = None
+
+
+    def __startTimer(self):
+        """ Start the timer, cancel the previous one if any """
+        self.__stopTimer()
+        self.timer = Timer(0.5, self.showNotification)
+        self.timer.start()
+
+
     # --== Message handler ==--
 
 
     def handleMsg(self, msg, params):
         """ Handle messages sent to this module """
         if msg == consts.MSG_EVT_NEW_TRACK and self.initialized:
-            self.showNotification(params['track'])
+            self.currCover = None
+            self.currTrack = params['track']
+            self.__startTimer()
         elif msg in (consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_MOD_LOADED):
             self.onModLoaded()
         elif msg in (consts.MSG_EVT_APP_QUIT, consts.MSG_EVT_MOD_UNLOADED, consts.MSG_EVT_STOPPED) and self.notif is not None:
+            self.currTrack = None
+            self.currCover = None
+            self.__stopTimer()
             self.notif.close()
         elif msg == consts.MSG_EVT_TRACK_MOVED:
             self.hasNext = params['hasNext']
+        elif msg == consts.MSG_CMD_SET_COVER and params['track'] == self.currTrack:
+            self.currCover = params['pathThumbnail']
 
 
     # --== Configuration ==--
