@@ -42,7 +42,7 @@ class Explorer(modules.Module):
 
     def __init__(self):
         """ Constructor """
-        modules.Module.__init__(self, (consts.MSG_CMD_EXPLORER_ADD, consts.MSG_CMD_EXPLORER_REMOVE, consts.MSG_EVT_APP_QUIT))
+        modules.Module.__init__(self, (consts.MSG_CMD_EXPLORER_ADD, consts.MSG_CMD_EXPLORER_REMOVE, consts.MSG_CMD_EXPLORER_RENAME))
         # Attributes
         self.combo        = prefs.getWidgetsTree().get_widget('combo-explorer')
         self.widgets      = {}
@@ -159,6 +159,31 @@ class Explorer(modules.Module):
         self.__fillComboBox()
 
 
+    def renameExplorer(self, modName, expName, newExpName):
+        """ Rename the given explorer """
+        # Beware of this special case: This screws the dictionnary
+        if newExpName == expName:
+            return
+
+        try:
+            # Modify the dictionnary
+            self.explorers[modName][newExpName] = self.explorers[modName][expName]
+            del self.explorers[modName][expName]
+
+            # Rename the combo box entry
+            for row in self.store:
+                if row[ROW_NAME] == expName:
+                    row[ROW_NAME] = newExpName
+                    break
+
+            # Changed the saved name if needed
+            (savedModName, savedExpName) = prefs.get(__name__, 'last-explorer', (modName, expName))
+            if savedModName == modName and savedExpName == expName:
+                prefs.set(__name__, 'last-explorer', (modName, newExpName))
+        except:
+            pass
+
+
    # --== Message handler ==--
 
 
@@ -168,6 +193,8 @@ class Explorer(modules.Module):
             self.addExplorer(params['modName'], params['expName'], params['icon'], params['widget'])
         elif msg == consts.MSG_CMD_EXPLORER_REMOVE:
             self.delExplorer(params['modName'], params['expName'])
+        elif msg == consts.MSG_CMD_EXPLORER_RENAME:
+            self.renameExplorer(params['modName'], params['expName'], params['newExpName'])
 
 
     # --== GTK handlers ==--
