@@ -426,7 +426,7 @@ class FileExplorer(modules.Module):
                 self.tree.connect('key-press-event', self.onKeyPressed)
                 self.tree.connect('exttreeview-button-pressed', self.onMouseButton)
                 self.tree.connect('exttreeview-row-collapsed', self.onRowCollapsed)
-                self.expandedHandler = self.tree.connect('exttreeview-row-expanded', self.onRowExpanded)
+                self.tree.connect('exttreeview-row-expanded', self.onRowExpanded)
 
             savedStates = prefs.get(__name__, 'saved-states', {})
 
@@ -450,9 +450,9 @@ class FileExplorer(modules.Module):
             else:
                 savedState = savedStates[newRoot]
 
-                self.tree.disconnect(self.expandedHandler)
+                self.tree.handler_block_by_func(self.onRowExpanded)
                 self.restoreTree(savedState['tree-state'])
-                self.expandedHandler  = self.tree.connect('exttreeview-row-expanded', self.onRowExpanded)
+                self.tree.handler_unblock_by_func(self.onRowExpanded)
 
                 idle_add(self.scrolled.get_vscrollbar().set_value, savedState['vscrollbar-pos'])
                 idle_add(self.scrolled.get_hscrollbar().set_value, savedState['hscrollbar-pos'])
@@ -534,12 +534,13 @@ class FileExplorer(modules.Module):
 
                 # Remove the tree, if any, from the scrolled window
                 if self.currRoot == name:
-                    self.scrolled.remove(self.trees[name][0])
                     self.currRoot = None
 
-                # Remove the tree associated to this root folder, if any
-                if name in self.trees:
-                    del self.trees[name]
+                # Remove the saved state of the tree, if any
+                savedStates = prefs.get(__name__, 'saved-states', {})
+                if name in savedStates:
+                    del savedStates[name]
+                    prefs.set(__name__, 'saved-states', savedStates)
 
             self.cfgList.removeSelectedRows()
 
