@@ -32,14 +32,17 @@ class GSTPlayer(modules.Module):
 
     def __init__(self):
         """ Constructor """
+        # The player must be created during the application startup, not when the application is ready (MSG_EVT_APP_STARTED)
+        self.player = audioplayer.AudioPlayer(self.__onTrackEnded, prefs.getCmdLine()[0].playbin2)
+
         modules.Module.__init__(self, (consts.MSG_CMD_PLAY,   consts.MSG_CMD_SET_VOLUME,   consts.MSG_CMD_ENABLE_RG,
                                        consts.MSG_CMD_SEEK,   consts.MSG_EVT_APP_STARTED,  consts.MSG_CMD_DISABLE_RG,
                                        consts.MSG_CMD_STOP,   consts.MSG_CMD_TOGGLE_PAUSE, consts.MSG_CMD_ENABLE_EQZ,
-                                       consts.MSG_CMD_BUFFER, consts.MSG_CMD_SET_EQZ_LVLS))
+                                       consts.MSG_CMD_BUFFER, consts.MSG_CMD_SET_EQZ_LVLS, consts.MSG_CMD_STEP,
+                                       consts.MSG_CMD_SET_CD_SPEED))
 
     def onAppStarted(self):
         """ This is the real initialization function, called when this module has been loaded """
-        self.player        = audioplayer.AudioPlayer(self.__onTrackEnded, prefs.getCmdLine()[0].playbin2)
         self.nextURI       = None
         self.queuedSeek    = None
         self.updateTimer   = None
@@ -159,6 +162,19 @@ class GSTPlayer(modules.Module):
         elif self.player.isPlaying(): self.player.seek(seconds*1000000000)
 
 
+    def step(self, seconds):
+        """ Step back or forth """
+        if self.player.isPlaying():
+            newPos = self.player.getPosition() + (seconds * 1000000000)
+
+            if newPos < 0:
+                self.player.seek(0)
+                self.updateTimerHandler()
+            elif newPos < self.player.getDuration():
+                self.player.seek(newPos)
+                self.updateTimerHandler()
+
+
     def setVolume(self, value):
         """ Change the volume """
         self.player.setVolume(value)
@@ -173,6 +189,7 @@ class GSTPlayer(modules.Module):
         if   msg == consts.MSG_CMD_STOP:         self.stop()
         elif msg == consts.MSG_CMD_PLAY:         self.play(params['uri'])
         elif msg == consts.MSG_CMD_SEEK:         self.seek(params['seconds'])
+        elif msg == consts.MSG_CMD_STEP:         self.step(params['seconds'])
         elif msg == consts.MSG_CMD_BUFFER:       self.bufferNextTrack(params['uri'])
         elif msg == consts.MSG_CMD_ENABLE_RG:    self.player.enableReplayGain()
         elif msg == consts.MSG_CMD_DISABLE_RG:   self.player.disableReplayGain()
@@ -181,3 +198,4 @@ class GSTPlayer(modules.Module):
         elif msg == consts.MSG_CMD_ENABLE_EQZ:   self.player.enableEqualizer()
         elif msg == consts.MSG_CMD_SET_EQZ_LVLS: self.player.setEqualizerLvls(params['lvls'])
         elif msg == consts.MSG_CMD_TOGGLE_PAUSE: self.togglePause()
+        elif msg == consts.MSG_CMD_SET_CD_SPEED: self.player.setCDReadSpeed(params['speed'])
