@@ -144,7 +144,9 @@ class Pidgin:
         """ Change the status message of the given account """
         try:
             current    = self.dbusInterface.PurpleSavedstatusGetCurrent()
-            statusType = dbus.UInt32(self.dbusInterface.PurpleSavedstatusGetType(current))
+            # This used to be needed, but seems to have been fixed in Pidgin
+            # statusType = dbus.UInt32(self.dbusInterface.PurpleSavedstatusGetType(current))
+            statusType = self.dbusInterface.PurpleSavedstatusGetType(current)
             statusId   = self.dbusInterface.PurplePrimitiveGetIdFromType(statusType)
 
             if statusId == 'available' or prefs.get(__name__, 'update-when-away', DEFAULT_UPDATE_WHEN_AWAY):
