@@ -33,7 +33,7 @@ MOD_NAME = MOD_INFO[modules.MODINFO_NAME]
 
 
 # Constants
-VERSION                  = 2                                      # Used to enforce compatibility
+VERSION                  = 3                                      # Used to enforce compatibility
 ROOT_PATH                = os.path.join(consts.dirCfg, 'Library') # Path where libraries are stored
 PREFS_DEFAULT_PREFIXES   = {'the ': None}                         # Prefixes are put at the end of artists' names
 PREFS_DEFAULT_LIBRARIES  = {}                                     # No libraries at first
@@ -79,11 +79,12 @@ PREFS_DEFAULT_TREE_STATE = {}                                     # No state at
 # The format of a row in the treeview
 (
     ROW_PIXBUF,    # Item icon
+    ROW_ALBUM_LEN, # Length of the album (invisible when not an album)
     ROW_NAME,      # Item name
     ROW_TYPE,      # The type of the item (e.g., directory, file)
     ROW_FULLPATH,  # The full path to the item
     ROW_TAGS       # The tags of the track, valid only for rows of type TYPE_TRACK
-) = range(5)
+) = range(6)
 
 
 class Library(modules.Module):
@@ -119,18 +120,29 @@ class Library(modules.Module):
 
     def __createTree(self):
         """ Create the main tree, add it to the scrolled window """
-        txtRdr    = gtk.CellRendererText()
-        pixbufRdr = gtk.CellRendererPixbuf()
+        txtRdr         = gtk.CellRendererText()
+        pixbufRdr      = gtk.CellRendererPixbuf()
+        txtRdrAlbumLen = gtk.CellRendererText()
 
-        columns = (('',   [(pixbufRdr, gtk.gdk.Pixbuf), (txtRdr, TYPE_STRING)], False),
-                   (None, [(None, TYPE_INT)],                                   False),
-                   (None, [(None, TYPE_STRING)],                                False),
-                   (None, [(None, TYPE_PYOBJECT)],                              False))
+        columns = (('',   [(pixbufRdr, gtk.gdk.Pixbuf), (txtRdrAlbumLen, TYPE_STRING), (txtRdr, TYPE_STRING)], False),
+                   (None, [(None, TYPE_INT)],                                                                  False),
+                   (None, [(None, TYPE_STRING)],                                                               False),
+                   (None, [(None, TYPE_PYOBJECT)],                                                             False))
 
         self.tree = extTreeview.ExtTreeView(columns, True)
 
-        self.tree.get_column(0).set_cell_data_func(txtRdr,    self.__drawCell)
-        self.tree.get_column(0).set_cell_data_func(pixbufRdr, self.__drawCell)
+        # The first text column (ROW_ALBUM_LEN) is not the one to search for
+        # set_search_column(ROW_NAME) should work, but it doesn't...
+        self.tree.set_search_equal_func(self.__searchFunc)
+
+        self.tree.get_column(0).set_cell_data_func(txtRdr,         self.__drawCell)
+        self.tree.get_column(0).set_cell_data_func(pixbufRdr,      self.__drawCell)
+        self.tree.get_column(0).set_cell_data_func(txtRdrAlbumLen, self.__drawAlbumLenCell)
+
+        # The album length is written in a smaller font, with a lighter color
+        txtRdrAlbumLen.set_property('scale', 0.85)
+        txtRdrAlbumLen.set_property('foreground', '#909090')
+
         self.tree.setIsDraggableFunc(self.isDraggable)
         self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
         # GTK handlers
@@ -142,12 +154,23 @@ class Library(modules.Module):
         self.scrolled.add(self.tree)
 
 
+    def __searchFunc(self, model, column, key, iter):
+        """ Check whether the given key matches the current candidate (iter) """
+        return model.get_value(iter, ROW_NAME)[:len(key)].lower() != key.lower()
+
+
     def __drawCell(self, column, cell, model, iter):
         """ Use a different background color for alphabetical headers """
         if model.get_value(iter, ROW_TYPE) == TYPE_HEADER: cell.set_property('cell-background-gdk', self.tree.style.bg[gtk.STATE_PRELIGHT])
         else:                                              cell.set_property('cell-background',     None)
 
 
+    def __drawAlbumLenCell(self, column, cell, model, iter):
+        """ Use a different background color for alphabetical headers """
+        if model.get_value(iter, ROW_ALBUM_LEN) is None: cell.set_property('visible', False)
+        else:                                            cell.set_property('visible', True)
+
+
     def __createEmptyLibrary(self, name):
         """ Create bootstrap files for a new library """
         # Make sure that the root directory of all libraries exists
@@ -275,7 +298,7 @@ class Library(modules.Module):
         overallNbArtists = len(db)
 
         # The 'artists' file contains all known artists with their index, the 'files' file contains the file structure of the root path
-        allArtists = sorted([(artist, str(indexArtist), len(db[artist])) for indexArtist, artist in enumerate(db)], key = lambda a: a[0])
+        allArtists = sorted([(artist, str(indexArtist), len(db[artist])) for indexArtist, artist in enumerate(db)], key = lambda a: a[0].lower())
         pickleSave(os.path.join(libPath, 'files'),   newLibrary)
         pickleSave(os.path.join(libPath, 'artists'), allArtists)
 
@@ -444,48 +467,48 @@ class Library(modules.Module):
         if not os.path.exists(os.path.join(path, 'VERSION_%u' % VERSION)):
             logger.error('[%s] Version number does not match, loading of library "%s" aborted' % (MOD_NAME, name))
             error = _('This library is deprecated, please refresh it.')
-            tree.replaceContent([(consts.icoError, error, TYPE_NONE, None, None)])
+            tree.replaceContent([(consts.icoError, None, error, TYPE_NONE, None, None)])
             return
 
         # Create the rows, with alphabetical header if needed
         for artist in pickleLoad(os.path.join(path, 'artists')):
 
-            if len(artist[ART_NAME]) != 0: currChar = unicode(artist[ART_NAME], errors='replace')[0]
+            if len(artist[ART_NAME]) != 0: currChar = unicode(artist[ART_NAME], errors='replace')[0].lower()
             else:                          currChar = prevChar
 
             if prevChar != currChar and not (prevChar.isdigit() and currChar.isdigit()):
                 prevChar = currChar
-                if currChar.isdigit(): rows.append((None, '<b>0 - 9</b>',         TYPE_HEADER, None, None))
-                else:                  rows.append((None, '<b>%s</b>' % currChar, TYPE_HEADER, None, None))
+                if currChar.isdigit(): rows.append((None, None, '<b>0 - 9</b>',                 TYPE_HEADER, None, None))
+                else:                  rows.append((None, None, '<b>%s</b>' % currChar.upper(), TYPE_HEADER, None, None))
 
-            rows.append((consts.icoDir, cgi.escape(artist[ART_NAME]), TYPE_ARTIST, os.path.join(path, artist[ART_INDEX]), None))
+            rows.append((consts.icoDir, None, cgi.escape(artist[ART_NAME]), TYPE_ARTIST, os.path.join(path, artist[ART_INDEX]), None))
 
         # Insert all rows, and then add a fake child to each artist
         tree.replaceContent(rows)
         for node in tree.iterChildren(None):
             if tree.getItem(node, ROW_TYPE) == TYPE_ARTIST:
-                tree.appendRow((None, '', TYPE_NONE, '', None), node)
+                tree.appendRow((None, None, '', TYPE_NONE, '', None), node)
 
 
     def loadAlbums(self, tree, node, fakeChild):
         """ Initial load of all albums of the given node, assuming it is of type TYPE_ARTIST """
         allAlbums = pickleLoad(os.path.join(tree.getItem(node, ROW_FULLPATH), 'albums'))
         path      = tree.getItem(node, ROW_FULLPATH)
-        rows      = [(consts.icoMediaDir, '<span size="smaller" foreground="#909090">[%s]</span>  %s' % (tools.sec2str(album[ALB_LENGTH], True), cgi.escape(album[ALB_NAME])), TYPE_ALBUM, os.path.join(path, album[ALB_INDEX]), None) for album in allAlbums]
+        rows      = [(consts.icoMediaDir, '[%s]' % tools.sec2str(album[ALB_LENGTH], True), '%s' % cgi.escape(album[ALB_NAME]), TYPE_ALBUM, os.path.join(path, album[ALB_INDEX]), None) for album in allAlbums]
 
         # Add all the rows, and then add a fake child to each of them
         tree.freeze_child_notify()
         tree.appendRows(rows, node)
         tree.removeRow(fakeChild)
         for child in tree.iterChildren(node):
-            tree.appendRow((None, '', TYPE_NONE, '', None), child)
+            tree.appendRow((None, None, '', TYPE_NONE, '', None), child)
         tree.thaw_child_notify()
 
 
     def loadTracks(self, tree, node, fakeChild):
         """ Initial load of all tracks of the given node, assuming it is of type TYPE_ALBUM """
         allTracks = pickleLoad(tree.getItem(node, ROW_FULLPATH))
-        rows      = [(consts.icoMediaFile, '%02u. %s' % (track.getNumber(), cgi.escape(track.getTitle())), TYPE_TRACK, track.getFilePath(), track) for track in allTracks]
+        rows      = [(consts.icoMediaFile, None, '%02u. %s' % (track.getNumber(), cgi.escape(track.getTitle())), TYPE_TRACK, track.getFilePath(), track) for track in allTracks]
 
         tree.appendRows(rows, node)
         tree.removeRow(fakeChild)
