@@ -26,23 +26,7 @@ from gui.extListview import ExtListView
 
 MOD_INFO = ('Tracklist', 'Tracklist', '', [], True, False)
 
-# Default preferences
-PREFS_DEFAULT_REPEAT_STATUS = False
-
-PREFS_DEFAULT_COLUMNS_VISIBILITY = {
-            '#'         : True,
-            _('Title')  : True,
-            _('Artist') : True,
-            _('Album')  : True,
-            _('Date')   : False,
-            _('Genre')  : False,
-            _('Length') : True,
-            _('Path')   : False,
-        }
-
-
-TRACKLIST_NB_FIELDS = 10
-
+# Create a unique ID for each field of a row in the list
 (
     ROW_ICO,   # Icon drawn in front of the row
     ROW_NUM,   # Track number
@@ -50,11 +34,38 @@ TRACKLIST_NB_FIELDS = 10
     ROW_ART,   # Track artist
     ROW_ALB,   # Track album
     ROW_LEN,   # Track length in seconds
+    ROW_BTR,   # Bit rate
     ROW_GNR,   # Genre
     ROW_DAT,   # Date
     ROW_PTH,   # Path to the file
     ROW_TRK    # The Track object
-) = range(TRACKLIST_NB_FIELDS)
+) = range(11)
+
+# Create a unique ID for each column that the user can actually see
+(
+    COL_TRCK_NUM,
+    COL_TITLE,
+    COL_ARTIST,
+    COL_ALBUM,
+    COL_DATE,
+    COL_GENRE,
+    COL_LENGTH,
+    COL_PATH,
+    COL_BITRATE,
+) = range(9)
+
+
+PREFS_DEFAULT_REPEAT_STATUS = False
+PREFS_DEFAULT_COLUMNS_VISIBILITY = { COL_TRCK_NUM : True,
+                                     COL_TITLE    : True,
+                                     COL_ARTIST   : True,
+                                     COL_ALBUM    : True,
+                                     COL_DATE     : False,
+                                     COL_GENRE    : False,
+                                     COL_LENGTH   : True,
+                                     COL_BITRATE  : False,
+                                     COL_PATH     : False,
+                                   }
 
 
 class Tracklist(modules.Module):
@@ -66,21 +77,20 @@ class Tracklist(modules.Module):
                                        consts.MSG_CMD_TOGGLE_PAUSE,      consts.MSG_CMD_NEXT,              consts.MSG_CMD_PREVIOUS,
                                        consts.MSG_EVT_NEED_BUFFER,       consts.MSG_EVT_STOPPED,           consts.MSG_EVT_PAUSED,
                                        consts.MSG_EVT_UNPAUSED,          consts.MSG_EVT_TRACK_ENDED_OK,    consts.MSG_EVT_TRACK_ENDED_ERROR,
-                                       consts.MSG_CMD_TRACKLIST_SHUFFLE, consts.MSG_EVT_APP_STARTED))
+                                       consts.MSG_CMD_TRACKLIST_SHUFFLE, consts.MSG_EVT_APP_STARTED,       consts.MSG_CMD_BRING_TO_FRONT))
 
 
     def onAppStarted(self):
         """ This is the real initialization function, called when the module has been loaded """
-        wTree         = tools.prefs.getWidgetsTree()
-        self.popup    = None
-        self.playtime = 0
+        wTree                  = tools.prefs.getWidgetsTree()
+        self.playtime          = 0
+        self.previousTracklist = None
         # Retrieve widgets
         self.window     = wTree.get_widget('win-main')
         self.btnClear   = wTree.get_widget('btn-tracklistClear')
         self.btnRepeat  = wTree.get_widget('btn-tracklistRepeat')
         self.btnShuffle = wTree.get_widget('btn-tracklistShuffle')
         self.btnClear.set_sensitive(False)
-        self.btnRepeat.set_sensitive(False)
         self.btnShuffle.set_sensitive(False)
         # Create the list and its columns
         txtLRdr   = gtk.CellRendererText()
@@ -88,22 +98,27 @@ class Tracklist(modules.Module):
         pixbufRdr = gtk.CellRendererPixbuf()
         txtRRdr.set_property('xalign', 1.0)
 
-        visible = tools.prefs.get(__name__, 'columns-visibility', PREFS_DEFAULT_COLUMNS_VISIBILITY)
+        # 'columns-visibility' may be broken, we should not use it (#311293)
+        visible = tools.prefs.get(__name__, 'columns-visibility-2', PREFS_DEFAULT_COLUMNS_VISIBILITY)
+        for (key, value) in PREFS_DEFAULT_COLUMNS_VISIBILITY.iteritems():
+            if key not in visible:
+                visible[key] = value
 
-        columns = (('#',         [(pixbufRdr, gtk.gdk.Pixbuf), (txtRRdr, TYPE_INT)], (ROW_NUM, ROW_TIT),                            False, visible['#']),
-                   (_('Title'),  [(txtLRdr, TYPE_STRING)],                           (ROW_TIT,),                                    True,  visible[_('Title')]),
-                   (_('Artist'), [(txtLRdr, TYPE_STRING)],                           (ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT),          True,  visible[_('Artist')]),
-                   (_('Album'),  [(txtLRdr, TYPE_STRING)],                           (ROW_ALB, ROW_NUM, ROW_TIT),                   True,  visible[_('Album')]),
-                   (_('Length'), [(txtRRdr, TYPE_INT)],                              (ROW_LEN,),                                    False, visible[_('Length')]),
-                   (_('Genre'),  [(txtLRdr, TYPE_STRING)],                           (ROW_GNR, ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), False, visible[_('Genre')]),
-                   (_('Date'),   [(txtLRdr, TYPE_INT)],                              (ROW_DAT, ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), False, visible[_('Date')]),
-                   (_('Path'),   [(txtLRdr, TYPE_STRING)],                           (ROW_PTH,),                                    False, visible[_('Path')]),
-                   (None,        [(None, TYPE_PYOBJECT)],                            (None,),                                       False, False))
+        columns = (('#',           [(pixbufRdr, gtk.gdk.Pixbuf), (txtRRdr, TYPE_INT)], (ROW_NUM, ROW_TIT),                            False, visible[COL_TRCK_NUM]),
+                   (_('Title'),    [(txtLRdr, TYPE_STRING)],                           (ROW_TIT,),                                    True,  visible[COL_TITLE]),
+                   (_('Artist'),   [(txtLRdr, TYPE_STRING)],                           (ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT),          True,  visible[COL_ARTIST]),
+                   (_('Album'),    [(txtLRdr, TYPE_STRING)],                           (ROW_ALB, ROW_NUM, ROW_TIT),                   True,  visible[COL_ALBUM]),
+                   (_('Length'),   [(txtRRdr, TYPE_INT)],                              (ROW_LEN,),                                    False, visible[COL_LENGTH]),
+                   (_('Bit Rate'), [(txtRRdr, TYPE_STRING)],                           (ROW_BTR, ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), False, visible[COL_BITRATE]),
+                   (_('Genre'),    [(txtLRdr, TYPE_STRING)],                           (ROW_GNR, ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), False, visible[COL_GENRE]),
+                   (_('Date'),     [(txtLRdr, TYPE_INT)],                              (ROW_DAT, ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), False, visible[COL_DATE]),
+                   (_('Path'),     [(txtLRdr, TYPE_STRING)],                           (ROW_PTH,),                                    False, visible[COL_PATH]),
+                   (None,          [(None, TYPE_PYOBJECT)],                            (None,),                                       False, False))
 
         self.list = ExtListView(columns, sortable=True, dndTargets=consts.DND_TARGETS.values(), useMarkup=False, canShowHideColumns=True)
         self.list.get_column(4).set_cell_data_func(txtRRdr, self.fmtLength)
         self.list.enableDNDReordering()
-        wTree.get_widget('scrolledwindow1').add(self.list)
+        wTree.get_widget('scrolled-tracklist').add(self.list)
         # GTK handlers
         self.list.connect('extlistview-dnd',                       self.onDND)
         self.list.connect('key-press-event',                       self.onKeyboard)
@@ -116,27 +131,44 @@ class Tracklist(modules.Module):
         self.btnShuffle.connect('clicked', lambda widget: modules.postMsg(consts.MSG_CMD_TRACKLIST_SHUFFLE))
         # Restore preferences
         self.btnRepeat.set_active(tools.prefs.get(__name__, 'repeat-status', PREFS_DEFAULT_REPEAT_STATUS))
-        # Try to associate an icon to the shuffle button
-        if consts.fileImgShuffle is not None: wTree.get_widget('img-shuffle').set_from_file(consts.fileImgShuffle)
-        else:                                 wTree.get_widget('img-shuffle').hide()
-        # Try to associate an icon to the repeat button
-        if consts.fileImgRepeat is not None: wTree.get_widget('img-repeat').set_from_file(consts.fileImgRepeat)
-        else:                                wTree.get_widget('img-repeat').hide()
+        # Set icons
+        wTree.get_widget('img-repeat').set_from_icon_name('stock_repeat', gtk.ICON_SIZE_BUTTON)
+        wTree.get_widget('img-shuffle').set_from_icon_name('stock_shuffle', gtk.ICON_SIZE_BUTTON)
 
 
-    # Helper functions
-    def hasNext(self):                      return self.list.hasMarkUnder(len(self.list)-1)
-    def hasPrev(self):                      return self.list.hasMarkAbove(0)
     def getAllFiles(self):                  return [row[ROW_TRK].getFilePath() for row in self.list.iterAllRows()]
     def getAllTracks(self):                 return [row[ROW_TRK] for row in self.list.iterAllRows()]
     def fmtLength(self, col, cll, mdl, it): cll.set_property('text', tools.sec2str(mdl.get_value(it, ROW_LEN)))
 
 
-    def __updateButtons(self, sensitive):
-        """ Enable/disable bottom buttons based on the sensitive parameter """
-        self.btnClear.set_sensitive(sensitive)
-        self.btnRepeat.set_sensitive(sensitive)
-        self.btnShuffle.set_sensitive(sensitive)
+    def __getNextTrackIdx(self):
+        """ Return the index of the next track, or -1 if there is none """
+        if self.list.hasMark():
+            if self.list.getMark() < (len(self.list) - 1): return self.list.getMark() + 1
+            elif self.btnRepeat.get_active():              return 0
+        return -1
+
+
+    def __getPreviousTrackIdx(self):
+        """ Return the index of the previous track, or -1 if there is none """
+        if self.list.hasMark():
+            if self.list.getMark() > 0:       return self.list.getMark() - 1
+            elif self.btnRepeat.get_active(): return len(self.list) - 1
+        return -1
+
+
+    def jumpToNext(self):
+        """ Jump to the next track, if any """
+        where = self.__getNextTrackIdx()
+        if where != -1:
+            self.jumpTo(where)
+
+
+    def jumpToPrevious(self):
+        """ Jump to the previous track, if any """
+        where = self.__getPreviousTrackIdx()
+        if where != -1:
+            self.jumpTo(where)
 
 
     def jumpTo(self, trackIdx):
@@ -148,7 +180,7 @@ class Tracklist(modules.Module):
         self.list.setItem(trackIdx, ROW_ICO, consts.icoPlay)
         modules.postMsg(consts.MSG_CMD_PLAY,        {'uri': self.list.getItem(trackIdx, ROW_TRK).getURI()})
         modules.postMsg(consts.MSG_EVT_NEW_TRACK,   {'track': self.list.getRow(trackIdx)[ROW_TRK]})
-        modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {'hasPrevious': self.hasPrev(), 'hasNext': self.hasNext() or self.btnRepeat.get_active()})
+        modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {'hasPrevious': self.__getPreviousTrackIdx() != -1, 'hasNext': self.__getNextTrackIdx() != -1})
 
 
     def onTrackEnded(self, withError):
@@ -175,38 +207,42 @@ class Tracklist(modules.Module):
 
     def onBufferingNeeded(self):
         """ The current track is close to its end, so we try to buffer the next one to avoid gaps """
-        if self.hasNext():
-            modules.postMsg(consts.MSG_CMD_BUFFER, {'uri': self.list.getItem(self.list.getMark()+1, ROW_TRK).getURI()})
-        elif self.btnRepeat.get_active():
-            modules.postMsg(consts.MSG_CMD_BUFFER, {'uri': self.list.getItem(0, ROW_TRK).getURI()})
+        where = self.__getNextTrackIdx()
+        if where != -1:
+            modules.postMsg(consts.MSG_CMD_BUFFER, {'uri': self.list.getItem(where, ROW_TRK).getURI()})
 
 
     def insert(self, tracks, position=None):
         """ Insert some tracks in the tracklist, append them if position is None """
+        self.previousTracklist = [row[ROW_TRK] for row in self.list.getAllRows()]
         rows = [[consts.icoNull, track.getNumber(), track.getTitle(), track.getArtist(), track.getExtendedAlbum(),
-                    track.getLength(), track.getGenre(), track.getDate(), track.getURI(), track] for track in tracks]
+                    track.getLength(), track.getBitrate(), track.getGenre(), track.getDate(), track.getURI(), track] for track in tracks]
 
         for row in rows:
             self.playtime += row[ROW_LEN]
 
-        self.__updateButtons(True)
         self.list.insertRows(rows, position)
 
 
     def set(self, tracks, playNow):
         """ Replace the tracklist, clear it if tracks is None """
-        self.playtime = 0
+        self.playtime     = 0
+        previousTracklist = [row[ROW_TRK] for row in self.list.getAllRows()]
+
+        if self.list.hasMark() and ((not playNow) or (tracks is None) or (len(tracks) == 0)):
+            modules.postMsg(consts.MSG_CMD_STOP)
+
         self.list.clear()
 
-        if tracks is None:
-            self.__updateButtons(False)
-            modules.postMsg(consts.MSG_CMD_STOP)
+        if tracks is None or len(tracks) == 0:
             modules.postMsg(consts.MSG_EVT_NEW_TRACKLIST, {'tracks': [], 'playtime': 0})
         else:
             self.insert(tracks)
             if playNow and len(self.list) != 0:
                 self.jumpTo(0)
 
+        self.previousTracklist = previousTracklist
+
 
     def onStopped(self):
         """ Playback has been stopped """
@@ -225,31 +261,38 @@ class Tracklist(modules.Module):
 
     def savePlaylist(self):
         """ Save the current tracklist to a playlist """
-        destDir = os.path.dirname(self.list.getItem(0, ROW_TRK).getFilePath())
-        outFile = gui.fileChooser.save(self.window, _('Save playlist'), 'playlist.m3u', destDir)
+        outFile = gui.fileChooser.save(self.window, _('Save playlist'), 'playlist.m3u')
 
         if outFile is not None:
             media.playlist.save(self.getAllFiles(), outFile)
 
 
-    def cropSelection(self):
-        """ Remove all tracks but the selection """
-        self.playtime = sum([row[ROW_LEN] for row in self.list.iterSelectedRows()])
-        self.list.cropSelectedRows()
+    def removeSelection(self, invert=False):
+        """ Remove the selected tracks if invert is False / the unselected tracks if invert is True """
+        hadMark                = self.list.hasMark()
+        selectionPlaytime      = sum([row[ROW_LEN] for row in self.list.iterSelectedRows()])
+        self.previousTracklist = [row[ROW_TRK] for row in self.list.getAllRows()]
+
+        if invert:
+            self.playtime = selectionPlaytime
+            self.list.cropSelectedRows()
+        else:
+            self.playtime -= selectionPlaytime
+            self.list.removeSelectedRows()
+
+        if hadMark and not self.list.hasMark():
+            modules.postMsg(consts.MSG_CMD_STOP)
 
 
-    def removeSelection(self):
-        """ Remove all the currently selected tracks """
-        for row in self.list.iterSelectedRows():
-            self.playtime -= row[ROW_LEN]
-            if row[ROW_ICO] == consts.icoPlay:
-                modules.postMsg(consts.MSG_CMD_STOP)
-
-        self.list.removeSelectedRows()
+    def revertTracklist(self):
+        """ Back to the previous tracklist """
+        self.set(self.previousTracklist, False)
+        self.previousTracklist = None
 
 
     def shuffleTracklist(self):
         """ Shuffle the tracks and ensure that the current track stays visible """
+        self.previousTracklist = [row[ROW_TRK] for row in self.list.getAllRows()]
         self.list.shuffle()
         if self.list.hasMark():
             self.list.scroll_to_cell(self.list.getMark())
@@ -257,20 +300,63 @@ class Tracklist(modules.Module):
 
     def showPopupMenu(self, list, path, button, time):
         """ The index parameter may be None """
-        if self.popup is None:
-            self.popup = tools.loadGladeFile('TracklistMenu.glade')
-            # Connect signals
-            self.popup.get_widget('item-save').connect('activate', lambda item: self.savePlaylist())
-            self.popup.get_widget('item-crop').connect('activate', lambda item: self.cropSelection())
-            self.popup.get_widget('item-remove').connect('activate', lambda item: self.removeSelection())
-            self.popup.get_widget('menu-popup').show_all()
+        popup = gtk.Menu()
 
-        # Enable only relevant menu entries
-        self.popup.get_widget('item-remove').set_sensitive(path is not None)
-        self.popup.get_widget('item-crop').set_sensitive(path is not None)
-        self.popup.get_widget('item-save').set_sensitive(len(list) != 0)
-        # Show the popup menu
-        self.popup.get_widget('menu-popup').popup(None, None, None, button, time)
+        # Crop
+        crop = gtk.ImageMenuItem(_('Crop'))
+        crop.set_image(gtk.image_new_from_stock(gtk.STOCK_CUT, gtk.ICON_SIZE_MENU))
+        crop.set_sensitive(path is not None)
+        crop.connect('activate', lambda item: self.removeSelection(True))
+        popup.append(crop)
+
+        # Remove
+        remove = gtk.ImageMenuItem(gtk.STOCK_REMOVE)
+        remove.set_sensitive(path is not None)
+        remove.connect('activate', lambda item: self.removeSelection())
+        popup.append(remove)
+
+        popup.append(gtk.SeparatorMenuItem())
+
+        # Shuffle
+        shuffle = gtk.ImageMenuItem(_('Shuffle Playlist'))
+        shuffle.set_sensitive(len(list) != 0)
+        shuffle.set_image(gtk.image_new_from_icon_name('stock_shuffle', gtk.ICON_SIZE_MENU))
+        shuffle.connect('activate', lambda item: modules.postMsg(consts.MSG_CMD_TRACKLIST_SHUFFLE))
+        popup.append(shuffle)
+
+        # Revert
+        revert = gtk.ImageMenuItem(_('Revert Playlist'))
+        revert.set_image(gtk.image_new_from_stock(gtk.STOCK_REVERT_TO_SAVED, gtk.ICON_SIZE_MENU))
+        revert.set_sensitive(self.previousTracklist is not None)
+        revert.connect('activate', lambda item: self.revertTracklist())
+        popup.append(revert)
+
+        # Clear
+        clear = gtk.ImageMenuItem(_('Clear Playlist'))
+        clear.set_sensitive(len(list) != 0)
+        clear.set_image(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU))
+        clear.connect('activate', lambda item: modules.postMsg(consts.MSG_CMD_TRACKLIST_CLR))
+        popup.append(clear)
+
+        popup.append(gtk.SeparatorMenuItem())
+
+        # Repeat
+        repeat = gtk.CheckMenuItem(_('Repeat'))
+        repeat.set_active(tools.prefs.get(__name__, 'repeat-status', PREFS_DEFAULT_REPEAT_STATUS))
+        repeat.connect('toggled', lambda item: self.btnRepeat.clicked())
+        popup.append(repeat)
+
+        popup.append(gtk.SeparatorMenuItem())
+
+        # Save
+        save = gtk.ImageMenuItem(_('Save Playlist As...'))
+        save.set_sensitive(len(list) != 0)
+        save.set_image(gtk.image_new_from_stock(gtk.STOCK_SAVE_AS, gtk.ICON_SIZE_MENU))
+        save.connect('activate', lambda item: self.savePlaylist())
+        popup.append(save)
+
+        popup.show_all()
+        popup.popup(None, None, None, button, time)
 
 
     # --== Message handler ==--
@@ -289,11 +375,14 @@ class Tracklist(modules.Module):
         elif msg == consts.MSG_CMD_TRACKLIST_SET:                            self.set(params['tracks'], params['playNow'])
         elif msg == consts.MSG_CMD_TRACKLIST_ADD:                            self.insert(params['tracks'])
         elif msg == consts.MSG_CMD_TRACKLIST_SHUFFLE:                        self.shuffleTracklist()
-        elif msg == consts.MSG_CMD_PREVIOUS and self.hasPrev():              self.jumpTo(self.list.getMark()-1)
-        elif msg == consts.MSG_CMD_TOGGLE_PAUSE and not self.list.hasMark(): self.jumpTo(0)
-        elif msg == consts.MSG_CMD_NEXT:
-            if self.hasNext():                self.jumpTo(self.list.getMark()+1)
-            elif self.btnRepeat.get_active(): self.jumpTo(0)
+        elif msg == consts.MSG_CMD_PREVIOUS:                                 self.jumpToPrevious()
+        elif msg == consts.MSG_CMD_NEXT:                                     self.jumpToNext()
+        elif msg == consts.MSG_CMD_BRING_TO_FRONT:                           self.window.present()
+        elif msg == consts.MSG_CMD_TOGGLE_PAUSE and not self.list.hasMark():
+            if self.list.getSelectedRowsCount() != 0:
+                self.jumpTo(self.list.getFirstSelectedRowIndex())
+            else:
+                self.jumpTo(0)
 
 
     # --== GTK handlers ==--
@@ -303,7 +392,7 @@ class Tracklist(modules.Module):
         """ The 'repeat' button has been pressed """
         tools.prefs.set(__name__, 'repeat-status', self.btnRepeat.get_active())
         if self.list.hasMark():
-            modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {'hasPrevious': self.hasPrev(), 'hasNext': self.hasNext() or self.btnRepeat.get_active()})
+            modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {'hasPrevious': self.__getPreviousTrackIdx() != -1, 'hasNext': self.__getNextTrackIdx() != -1})
 
 
     def onButtonPressed(self, list, event, path):
@@ -315,14 +404,21 @@ class Tracklist(modules.Module):
 
 
     def onKeyboard(self, list, event):
-        """ Remove the selection if possible """
-        if gtk.gdk.keyval_name(event.keyval) == 'Delete':
-            self.removeSelection()
+        """ Keyboard shortcuts """
+        keyname = gtk.gdk.keyval_name(event.keyval)
+
+        if keyname == 'Delete':   self.removeSelection()
+        elif keyname == 'Return': self.jumpTo(self.list.getFirstSelectedRowIndex())
+        elif keyname == 'space':  modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE)
+        elif keyname == 'Escape': modules.postMsg(consts.MSG_CMD_STOP)
+        elif keyname == 'Left':   modules.postMsg(consts.MSG_CMD_STEP, {'seconds': -5})
+        elif keyname == 'Right':  modules.postMsg(consts.MSG_CMD_STEP, {'seconds': 5})
 
 
     def onListModified(self, list):
         """ Some rows have been added/removed/moved """
-        self.__updateButtons(len(list) != 0)
+        self.btnClear.set_sensitive(len(list) != 0)
+        self.btnShuffle.set_sensitive(len(list) != 0)
 
         # Update playlist length and playlist position for all tracks
         for position, row in enumerate(self.list.getAllRows()):
@@ -330,14 +426,24 @@ class Tracklist(modules.Module):
             row[ROW_TRK].setPlaylistLen(len(self.list))
 
         modules.postMsg(consts.MSG_EVT_NEW_TRACKLIST, {'tracks': self.getAllTracks(), 'playtime': self.playtime})
-        modules.postMsg(consts.MSG_EVT_TRACK_MOVED,   {'hasPrevious': self.hasPrev(), 'hasNext':  self.hasNext()})
+        modules.postMsg(consts.MSG_EVT_TRACK_MOVED,   {'hasPrevious': self.__getPreviousTrackIdx() != -1, 'hasNext':  self.__getNextTrackIdx() != -1})
 
 
     def onColumnVisibilityChanged(self, list, colTitle, visible):
         """ A column has been shown/hidden """
-        visibility = tools.prefs.get(__name__, 'columns-visibility', PREFS_DEFAULT_COLUMNS_VISIBILITY)
-        visibility[colTitle] = visible
-        tools.prefs.set(__name__, 'columns-visibility', visibility)
+        if colTitle == '#':             colId = COL_TRCK_NUM
+        elif colTitle == _('Title'):    colId = COL_TITLE
+        elif colTitle == _('Artist'):   colId = COL_ARTIST
+        elif colTitle == _('Album'):    colId = COL_ALBUM
+        elif colTitle == _('Length'):   colId = COL_LENGTH
+        elif colTitle == _('Genre'):    colId = COL_GENRE
+        elif colTitle == _('Date'):     colId = COL_DATE
+        elif colTitle == _('Bit Rate'): colId = COL_BITRATE
+        else:                           colId = COL_PATH
+
+        visibility = tools.prefs.get(__name__, 'columns-visibility-2', PREFS_DEFAULT_COLUMNS_VISIBILITY)
+        visibility[colId] = visible
+        tools.prefs.set(__name__, 'columns-visibility-2', visibility)
 
 
     def onDND(self, list, context, x, y, dragData, dndId, time):
@@ -346,8 +452,6 @@ class Tracklist(modules.Module):
             context.finish(False, False, time)
             return
 
-        dropInfo = list.get_dest_row_at_pos(x, y)
-
         # A list of filenames, without 'file://' at the beginning
         if dndId == consts.DND_DAP_URI:
             tracks = media.getTracks([urllib.url2pathname(uri) for uri in dragData.data.split()])
@@ -358,7 +462,11 @@ class Tracklist(modules.Module):
         elif dndId == consts.DND_DAP_TRACKS:
             tracks = [media.track.unserialize(serialTrack) for serialTrack in dragData.data.split('\n')]
 
-        if dropInfo is None: self.insert(tracks)
-        else:                self.insert(tracks, dropInfo[0][0])
+        dropInfo = list.get_dest_row_at_pos(x, y)
+
+        # Insert the tracks, but beware of the AFTER/BEFORE mechanism used by GTK
+        if dropInfo is None:                          self.insert(tracks)
+        elif dropInfo[1] == gtk.TREE_VIEW_DROP_AFTER: self.insert(tracks, dropInfo[0][0] + 1)
+        else:                                         self.insert(tracks, dropInfo[0][0])
 
         context.finish(True, False, time)
