@@ -29,7 +29,7 @@ socketTimeout = 10
 
 # --- Strings
 appName      = 'Decibel Audio Player'
-appVersion   = '1.02'
+appVersion   = '1.04'
 appNameShort = 'decibel-audio-player'
 
 
@@ -97,8 +97,11 @@ UNKNOWN_TITLE        = _('Unknown Title')
 UNKNOWN_ALBUM        = _('Unknown Album')
 UNKNOWN_ARTIST       = _('Unknown Artist')
 UNKNOWN_LENGTH       = 0
+UNKNOWN_BITRATE      = 0
+UNKNOWN_ENC_MODE     = 0
 UNKNOWN_MB_TRACKID   = 0
 UNKNOWN_DISC_NUMBER  = 0
+UNKNOWN_SAMPLE_RATE  = 0
 UNKNOWN_TRACK_NUMBER = 0
 UNKNOWN_ALBUM_ARTIST = _('Unknown Album Artist')
 
@@ -141,9 +144,8 @@ DND_TARGETS = {
 (
     VIEW_MODE_FULL,
     VIEW_MODE_MINI,
-    VIEW_MODE_NANO,
     VIEW_MODE_PLAYLIST,
-) = range(4)
+) = range(3)
 
 
 # --- Message that can be sent/received by modules
@@ -155,6 +157,7 @@ DND_TARGETS = {
     MSG_CMD_PLAY,             # Play a resource                            Parameters: 'uri'
     MSG_CMD_STOP,             # Stop playing                               Parameters:
     MSG_CMD_SEEK,             # Jump to a position                         Parameters: 'seconds'
+    MSG_CMD_STEP,             # Step back or forth                         Parameters: 'seconds'
     MSG_CMD_SET_VOLUME,       # Change the volume                          Parameters: 'value'
     MSG_CMD_BUFFER,           # Buffer a file                              Parameters: 'filename'
     MSG_CMD_TOGGLE_PAUSE,     # Toggle play/pause                          Parameters:
@@ -162,6 +165,7 @@ DND_TARGETS = {
     MSG_CMD_SET_EQZ_LVLS,     # Set the levels of the 10-bands equalizer   Parameters: 'lvls'
     MSG_CMD_ENABLE_RG,        # Enable ReplayGain                          Parameters:
     MSG_CMD_DISABLE_RG,       # Disable ReplayGain                         Parameters:
+    MSG_CMD_SET_CD_SPEED,     # Change drive speed when reading a CD       Parameters: 'speed'
 
     # Tracklist
     MSG_CMD_NEXT,              # Play the next track       Parameters:
@@ -179,6 +183,10 @@ DND_TARGETS = {
     # Covers
     MSG_CMD_SET_COVER,         # Cover file for the given track     Parameters: 'track', 'pathThumbnail', 'pathFullSize'
 
+    # Misc
+    MSG_CMD_BRING_TO_FRONT,    # Bring the window to the front      Parameters:
+
+
     # --== EVENTS ==--
 
     # Current track
@@ -211,4 +219,4 @@ DND_TARGETS = {
 
     # End value
     MSG_END_VALUE
-) = range(37)
+) = range(40)
